function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, varargin)

% function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, (sampleweights))
%
% Christian Merkwirth 2002

global entooldrawit ssites

if isempty(trainparams)
    trainparams = get(model, 'trainparams');
end

nr_samples = length(bilder);
D = size(urbilder, 2);

if nargin > 6
    sampleweights = varargin{1};
else
    sampleweights = ones(nr_samples,1);
end  

[indtrain, indtest] = dissemble(sampleclass, nr_samples);

nr_train = length(indtrain);
nr_test = length(indtest);

u = urbilder(indtrain, :);
b = bilder(indtrain);

tiny = 1e-10;

trainerr = [];
testerr = [];

model.coefficients = [0];
model.splitdims = {};
model.splitsites = {};
model.splitdirs = {};

ssites = sort(u);       % Matrix of all possible splitsites

err = inf;

% Forward selection

x = [];

for depth=1:trainparams.maxdepth
    basisfunctionlist = create_list(u, model, trainparams);
    
    tmperr = ones(1,size(basisfunctionlist,2));
    tmpw = zeros(length(model.coefficients)+1, size(basisfunctionlist,2));
    
    w = [model.coefficients ; 0];
    
    for i=1:size(basisfunctionlist,2)    
        xn = create_basisfunction(u, basisfunctionlist{1,i}, basisfunctionlist{2,i}, basisfunctionlist{3,i}); 
        X = [ones(nr_train,1) x xn];
        [w,flag,relres] = lsqr(X, bilder(indtrain), [], 2*D, [],[], w);
        tmperr(i) = relres; % epsinloss(bilder(indtrain), yhat, eps);
        tmpw(:,i) = w;
    end
    
    [newerr, newbaseind] = min(tmperr);
    
    %disp(newerr)
    
    if (newerr >= err(end)) | (abs(err(end)-newerr) < 1e-4)  
        break;
    end
        
    %model.coefficients = tmpw(:,newbaseind);
    model.splitdims{end+1,1} = basisfunctionlist{1, newbaseind};
    model.splitsites{end+1,1} = basisfunctionlist{2, newbaseind};
    model.splitdirs{end+1,1} =  basisfunctionlist{3, newbaseind};
    
    % Also add reflected partner
    dirs = basisfunctionlist{3, newbaseind};
    dirs(end) = -dirs(end);    
    
    model.splitdims{end+1,1} = basisfunctionlist{1, newbaseind};
    model.splitsites{end+1,1} = basisfunctionlist{2, newbaseind};
    model.splitdirs{end+1,1} =  dirs;

    xn = create_basisfunction(u, basisfunctionlist{1, newbaseind}, ...
        basisfunctionlist{2, newbaseind}, basisfunctionlist{3, newbaseind}); 
        
    xn2 = create_basisfunction(u, basisfunctionlist{1, newbaseind}, ...
        basisfunctionlist{2, newbaseind}, dirs); 
    
    x = [x xn xn2];
   
    [model.coefficients,yh] = lreg([ones(nr_train,1) x], bilder(indtrain));
    %relepsinloss(bilder(indtrain), yh, eps);
        
    err(end+1) = newerr;
    %disp(newerr)
    
    solltrain = bilder(indtrain);
    isttrain = calc(model, urbilder(indtrain,:));
    trainerr(end+1) = relepsinloss(solltrain, isttrain, eps);
    
    if nr_test
        solltest = bilder(indtest);
        isttest = calc(model, urbilder(indtest,:));
        testerr(end+1) = relepsinloss(solltest, isttest, eps);   
    end
    
    if entooldrawit      
        subplot(1,2,1)
        plot(solltrain, isttrain, '.')
        hold on
        if nr_test
            plot(solltest, isttest, 'r.')    
        end
        hold off
        
        subplot(1,2,2)
        plot(trainerr)
        hold on
        if nr_test
            plot(testerr, 'r')
        end
        hold off
        title(num2str(trainerr(end))) 
        drawnow       
    end
    
    if newerr == 0
        break;
    end    
end  

% Backward deletion

models = {model};
gcvs = [gcv(model, urbilder(indtrain,:), bilder(indtrain), 2.5, eps)];

for j=1:length(model.splitdims)
    tmperr = [];
    tmpw = [];
    
    for k=1:length(model.splitdims)    
        tmpx = x;
        tmpx(:,k) = [];
        
        [tmpw(:,end+1),yh] = lreg([ones(nr_train,1) tmpx], bilder(indtrain));        
        tmperr(1, end+1) = epsinloss(bilder(indtrain), yh, eps); 
    end    
    
    [dummy, ind] = min(tmperr);
    
    x(:,ind) = [];
    
    model.coefficients = tmpw(:,ind);
    model.splitdims(ind) = [];
    model.splitsites(ind) = [];
    model.splitdirs(ind) = [];
   
    models{end+1} = model;
    
    [g, err] = gcv(model, urbilder(indtrain,:), bilder(indtrain), 2.5, eps);
    %disp([dummy err])
    
    gcvs(end+1) = g;
end    

[dummy, ind] = min(gcvs);
model = models{ind};



model.trainparams = trainparams;
model.eps = eps;

function [g, err] = gcv(model, u, b, c, eps)

z = calc(model, u);
err = epsinloss(b, z, eps); 

M = length(model.splitdims) + c * length(unique(cell2mat(model.splitsites')));    % FIXME : Not necessarily right number of knots

m = 1 - M/length(b);

if m <= 0
    g = inf;
else
    g = err / (m*m);
end





function basisfunctionlist = create_list(u, model, trainparams)

global ssites 

[N,D] = size(u);

nr_rand_splits = ceil(sqrt(N));         % FIXME : Make adaptable

basisfunctionlist = cell(3, 2*D*nr_rand_splits);

counter = 1;

for d=1:D
    randsplits = sort(randsel(ssites(:,d), nr_rand_splits));
    for i=1:length(randsplits)
        basisfunctionlist{1,counter} = d;
        basisfunctionlist{2,counter} = randsplits(i);
        basisfunctionlist{3,counter} = 1;
        counter = counter + 1;
    end
    for i=1:length(randsplits)
        basisfunctionlist{1,counter} = d;
        basisfunctionlist{2,counter} = randsplits(i);
        basisfunctionlist{3,counter} = -1;
        counter = counter + 1;
    end
end

for j=1:length(model.splitdims)
    if length(model.splitdims{j}) < trainparams.max_interactions+1
        alloweddims = 1:D;
        if ~trainparams.selfinteractions
            alloweddims = setdiff(alloweddims, model.splitdims{j}); 
        end    
        for d=alloweddims
            randsplits = sort(randsel(ssites(:,d), nr_rand_splits));
            for i=1:length(randsplits)
                basisfunctionlist{1, counter} = [model.splitdims{j} d];
                basisfunctionlist{2, counter} = [model.splitsites{j} randsplits(i)];
                basisfunctionlist{3, counter} = [model.splitdirs{j} 1];
                counter = counter + 1;
            end
            for i=1:length(randsplits)
                basisfunctionlist{1, counter} = [model.splitdims{j} d];
                basisfunctionlist{2, counter} = [model.splitsites{j} randsplits(i)];
                basisfunctionlist{3, counter} = [model.splitdirs{j} -1];
                counter = counter + 1;
            end
        end       
    end        
end        
        
    

function [w,yh] = lreg(x, y)

A = pinv(x'*x);
w = A*x'*y;
yh = x*w;
    



function [w,yh] = ridgereg(x, y, lambda)

x  = x - mean(x);

A = pinv(x'*x);
w = A*x'*y;
yh = x*w;
    
