function [z, testflag] = outoftraincalc(model, varargin)

% [z, testflag] = outoftraincalc(model, urbilder)
%
% Calculate the output of the ensemble for the input data set, but only
% average over those samples that were in the left-out part of the input
% data while training one of the models for the ensemble.
%
% To avoid that not all samples are actually test samples, check testflag
% for containing all ones. An entry of zero in testflag indicates that the
% output z was taken from training samples and are therefore might not
% be a good estimator for the test error

% Christian Merkwirth MPI 2002

if nargin < 2
    urbilder = model.urbilder;
else
    urbilder = varargin{1};    
end

s = sum(model.sampleclasses,2);
testflag = double(s > 0);

[z,zm] = calc(model, urbilder);

indno = find(s == 0);
s(indno) = 1;       % avoid division by zero
z = sum(zm.*model.sampleclasses, 2) ./ s;

% difficult : if there are no test samples for one column of z, use
% training samples to compute mean
z(indno) = mean(zm(indno,:), 2);

