function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, varargin)

% function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, varargin)
%
%
% Christian Merkwirth 2002

global entooldrawit

tiny = 1e-10;

if isempty(trainparams)
    trainparams = get(model, 'trainparams');
end

nr_samples = length(bilder);
nr_model_classes = size(trainparams.modelclasses,1);
[indtrain, indtest] = dissemble(sampleclass, nr_samples);

model.urbilder = urbilder(indtrain,:);
model.bilder = bilder(indtrain);

if trainparams.scaledata
    [model.ensemble, urbilder, bilder] = compute_scalefactors(model.ensemble, urbilder, bilder);    
end

urbilder = urbilder(indtrain,:);
bilder = bilder(indtrain);

msampleclass = getsampleclass(bilder, trainparams.nr_cv_partitions*nr_model_classes, trainparams.frac_test);

counter = 1;

for i=1:trainparams.nr_cv_partitions  
    for j=1:nr_model_classes 
        [thismodeltrain,thismodeltest] = dissemble(msampleclass(:,counter), length(bilder));
        
        if entooldrawit
            disp(['Cross-validation training on train/test partition ' num2str(counter)])    
        end   
       
        modelclass = trainparams.modelclasses{j,1};
        mtrainparams = trainparams.modelclasses{j,2};
        minitparams = trainparams.modelclasses{j,3}; 
        
        m = feval(modelclass, minitparams{:});
        
        if isempty(mtrainparams)
            mtrainparams = get(m, 'trainparams');
        end
        
        try  
            m = train(m, urbilder, bilder, msampleclass(:,counter), mtrainparams, eps);
        catch
            disp(['An error occured while training ' modelclass ', model will be discarded']);
            disp(lasterr)
            m = [];
        end
        
        if ~isempty(m)
            z = calc(m, urbilder);
            merr = relepsinloss(bilder, z, eps);
            model.ensemble = addmodel(model.ensemble, m, 1.0, merr);
            model.sampleclasses = [model.sampleclasses msampleclass(:,counter)];
            
            disp(['Trained ' modelclass ' model with total err ' ...
                     	num2str([merr])])   
        end
        
        counter = counter + 1;
    end
end

model = set(model, 'eps', eps);
model = set(model, 'trainparams', trainparams);

