function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, varargin)

% function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, (sampleweights))
%
%
% Christian Merkwirth 2002

global entooldrawit

if isempty(trainparams)
   trainparams = get(model, 'trainparams');
end

nr_train = length(bilder);

if nargin > 6
   sampleweights = varargin{1};
else
   sampleweights = ones(nr_train,1);
end  

[indtrain, indtest] = dissemble(sampleclass, nr_train);

Ntrain = length(indtrain);
Ntest = length(indtest);

tiny = 1e-10;

coeff = [];

sw = sampleweights(indtrain);
sw = sw / sum(sw);

trainerr = [];
testerr = [];

if trainparams.frac_leave_out == 0
	trainparams.repetitions = 1;	% no leave out part, so there's no need to repeat training
end   

Nleaveout = floor(Ntrain*trainparams.frac_leave_out);

for rounds=1:trainparams.repetitions
   indleaveout = randsel(1:length(indtrain), Nleaveout);
   
   ind = indtrain;
   ind(indleaveout) = [];
   
   w = linreg(urbilder(ind,:), bilder(ind));
   
   coeff(end+1,:) = w(:)';
   model.coefficients = w(:)';
   
   solltrain = bilder(indtrain);
   isttrain = calc(model, urbilder(indtrain,:));
   trainerr(end+1,1) = relepsinloss(solltrain, isttrain, eps, sw);
   
   if Ntest
   	solltest = bilder(indtest);
      isttest = calc(model, urbilder(indtest,:));
      testerr(end+1,1) = relepsinloss(solltest, isttest, eps, sampleweights(indtest));
   end
      
   if entooldrawit      
      subplot(2,2,1)
      plot(solltrain, isttrain, '.')
      hold on
      if Ntest
         plot(solltest, isttest, 'r.')
      end   
      hold off
      
      subplot(2,2,2)
      
      if length(trainerr) > 1
      	plot(trainerr)
         hold on
         if Ntest
            plot(testerr, 'r')
         end   
         hold off
      end   
      
      subplot(2,2,4)
      imagesc(coeff);	
      colorbar
      

      drawnow       
   end   
end

% Remove worst models

if trainparams.remove_worst
	[dummy,ind] = sort(-trainerr);   
   ind = ind(1:floor(trainparams.remove_worst*trainparams.repetitions));
   coeff(ind,:) = [];
   trainerr(ind) = [];
end

model.coefficients = mean(coeff, 1);

solltrain = bilder(indtrain);
isttrain = calc(model, urbilder(indtrain,:));

trainerr = relepsinloss(solltrain, isttrain, eps, sw);

model.trainparams = trainparams;
model.eps = eps;
