function [ model, trainerr ] = train( model, urbilder, bilder, sampleclass, trainparams, eps, varargin)

% function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, varargin)
%
% Christian Merkwirth 2002

global entooldrawit

if isempty(trainparams)
	trainparams = get(model, 'trainparams');
end
  
nr_samples = length(bilder);
nr_inputs = size(urbilder, 2);

[indtrain, indtest] = dissemble(sampleclass, nr_samples);

Ntrain = length(indtrain);
Ntest = length(indtest);

[urbilderscaled, uscalefacs] = scale(urbilder);
[bilderscaled, bscalefac] = scale(bilder);

urbilderscaled = trainparams.range * urbilderscaled;
bilderscaled = trainparams.range * bilderscaled;

model.uscalefacs = uscalefacs;
model.bscalefac = bscalefac;
   
model.trainparams = trainparams;

tiny = 1e-10;

trainerr = [];
testerr = [];

model.W1 = triangle_rand(model.topology, nr_inputs+1) / sqrt(nr_inputs+1);
model.W1 = triangle_rand(1, model.topology+1) / sqrt(model.topology+1);

conf = trainparams.conf;
   
[model.W1,model.W2, critvec, it, lambda] = marq(model.netstruct, model.W1, model.W2, urbilderscaled(indtrain,:)', ...
	bilderscaled(indtrain)', conf);

solltrain = bilder(indtrain);
isttrain = calc(model, urbilder(indtrain,:));
trainerr(end+1,1) = relepsinloss(solltrain, isttrain, eps);
   
if Ntest   
	solltest = bilder(indtest);
	isttest = calc(model, urbilder(indtest,:));
   
	testerr(end+1,1) = relepsinloss(solltest, isttest, eps);
end   

if entooldrawit      
     	subplot(2,2,1)
     	plot(solltrain, isttrain, '.') 
      hold on
      if Ntest
      	plot(solltest, isttest, 'r.') 
      end 
      hold off
      
      subplot(2,2,2)
     	plot(critvec) 
     
      subplot(2,2,3)
      plot(1:Ntrain,solltrain);
      hold on
      plot(1:Ntrain,isttrain, 'r');
      
      if Ntest 
      	plot((Ntrain+1):(Ntrain+Ntest), solltest, 'g');
      	plot((Ntrain+1):(Ntrain+Ntest), isttest, 'r');      
      end   
      
      hold off   
      drawnow
end   

model.trainparams = trainparams;
model.eps = eps;