function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, varargin)

% function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, (sampleweights))
%
%
% Christian Merkwirth 2002

global entooldrawit

if isempty(trainparams)
   trainparams = get(model, 'trainparams');
end

nr_samples = length(bilder);

if nargin > 6
   sampleweights = varargin{1};
else
   sampleweights = ones(nr_samples,1);
end  

[indtrain, indtest] = dissemble(sampleclass, nr_samples);

nr_train = length(indtrain);
nr_test = length(indtest);

model.trainparams = trainparams;
model.eps = eps;

tiny = 1e-10;

urbildertrain = urbilder(indtrain,:);
bildertrain = bilder(indtrain);

[N,D] = size(urbildertrain);

monomlist = mlist(D, 1)'; 

for grad = 2:model.maxgrad   
	tmpmonomlist = mlist(D, grad)';
  	tmpnr_monoms = size(monomlist, 1);	% total number of monoms up to degree grad
   
   if tmpnr_monoms > model.maxnrmonoms   
      break;
   else
      monomlist = tmpmonomlist;
   end   
end

monomlist(find(all(monomlist == 0,2)),:) = [];
nr_monoms = size(monomlist, 1);
disp(['Total number of monoms : ' num2str(nr_monoms)]);

sw = sampleweights(indtrain);
sw = sw / sum(sw);

trainerr = [];
testerr = [];

A = [];
monomindices = [];

err = inf;

mode = 0;	 
avoidindices = [];

for rounds=1:trainparams.maxrounds      
   if rounds==trainparams.maxrounds
      mode = 2;
   end   
   
   switch(mode)
   	case 0
         newind = [];
   		newcoeff = [];
   		newA = [];
         
         availindices = setdiff(1:nr_monoms, [avoidindices monomindices]);
         
   		for i=availindices
   			newmonomindices = [monomindices i];   
            Atmp = [A monomvalues(monomlist(i,:), urbildertrain)];
            
				[coeff, dummy, out] = linreg(Atmp, bildertrain);
            newerr = relepsinloss(bildertrain , out, eps);
            
            if newerr < err

           			newind = i;
         			err = newerr;
         			newcoeff = coeff';
               	newA = Atmp;  
      		end   
   		end   
   
   		if ~isempty(newind)
   			monomindices = [monomindices newind]; 
  				model.monoms = monomlist(monomindices,:);
            model.coefficients = newcoeff;
            A = newA;  
   		else
   			mode = 1;   
         end
      case 1
     		newcoeff = [];
   		newA = [];
         
    		for i=1:length(monomindices)
            tmpmonomindices = monomindices;
            tmpmonomindices(i) = [];
            
            Atmp = A;
            Atmp(:,i) = [];
            
				[coeff, dummy, out] = linreg(Atmp, bildertrain);
            newerr = relepsinloss(bildertrain , out, eps);
            
            if newerr < err
               newmonomindices = tmpmonomindices;
           		err = newerr;
         		newcoeff = coeff';
         		newA = Atmp;
      		end   
   		end   
   
   		if isempty(newcoeff)
      		mode = 2; 
   		else
   			monomindices = newmonomindices; 
  				model.monoms = monomlist(monomindices,:);
            model.coefficients = newcoeff;
            A = newA;   
         end   
      case 2
         s = sum(abs(model.coefficients(1:(end-1))));	% last coefficient is offset, ignore when pruning
         ind = find(abs(model.coefficients(1:(end-1))) < (trainparams.suppress * s));
                  
         if ~isempty(ind)
            avoidindices = ind;
            
         	A(:,ind) = [];
         	monomindices(ind) = [];
            
            [coeff, dummy, out] = linreg(A, bildertrain);
            err = relepsinloss(bildertrain , out, eps);

            
         	model.coefficients = coeff';
         	model.monoms = monomlist(monomindices,:);
         else
            avoidindices = [];
         end
         
         mode = 0;
   end
   
   solltrain = bilder(indtrain);
   isttrain = calc(model, urbilder(indtrain,:));
   trainerr(end+1,1) = relepsinloss(solltrain, isttrain, eps, sw);
   
   if nr_test
   	solltest = bilder(indtest);
   	isttest = calc(model, urbilder(indtest,:));
   	testerr(end+1,1) = relepsinloss(solltest, isttest, eps, sampleweights(indtest));
   end
   
   
   if entooldrawit      
      subplot(1,2,1)
      plot(solltrain, isttrain, '.')
      hold on
      if nr_test
      	plot(solltest, isttest, 'r.') 
      end   
      hold off
      
      subplot(1,2,2)
      plot(trainerr);
      hold on
      if nr_test
      	plot(testerr, 'r');
      end   
      hold off
      drawnow
   end   
end

% Sort polynoms for convenience
%[dummy,ind] = sortrows(model.monoms);
%model.monoms = model.monoms(ind,:);
%model.coefficients = model.coefficients(ind);
