function [y, z, a1, a2] = prbfn_out(p, rr_net, comp_p1)
% [y, z] = prbfn_out(p, rr_net,comp_p1)
%
% Description: this function computes the outputs of the
% nueral net which is composed of Ridge and Rbfs.
% Two outputs matrix are computed, one for the hidden
% layer and the other for the output layer.
%
% Inputs:
% p - a matrix of patterns. each pattern is a column 
% vector.
% rr_net - the ridge rbf net parameters as returned from
% ridge_rbf_train.
% comp_p1 - compute p1 addition of bias to input matrix,
% set comp_p1 to 1 when testing.
% Outputs:
% y - the output of the output layer.
% z - the ouput of the hidden layer.
% a1 - scalar product of firt layer.
% a2 - scalar product of second layer.
%
% Author: Shimon Cohen
% Copyrights (C) Shimon Cohen

[dim, npats] = size(p);
% number of rbfs
nrbfs = rr_net.nrbfs;
nridges = rr_net.nridges;
if (nargin <= 2)
    comp_p1 =1;
end

rads = 2 * rr_net.rads .* rr_net.rads;
if (rr_net.vec_rad == 0 & nrbfs > 0)
   rads = ones(dim,1) * rads;
end

z = ones(npats, nrbfs+nridges+1);
[nout, nhid] = size(rr_net.w2);

% a1 columns are activation per pattern
if (comp_p1 == 0)
   p1 = rr_net.p1;
else
   p1 = [ones(1, npats); p];
end
if (nridges > 0)
   if (isfield(rr_net,'w1_flgs'))
      a1 = (rr_net.w1 .* rr_net.w1_flgs) * p1;
   else
       a1 = rr_net.w1 * p1;  
   end
else
   a1 = [];
end

%rbf_ones = ones(1, nrbfs);

% computes output of rbfs
if (nrbfs > 0)
   npats_ones = ones(1, npats);
   for j=1:nrbfs
      dist = p - rr_net.m(:,j)*npats_ones;
      dist = dist.*dist;
      rd = rads(:,j) * npats_ones;
      if (rr_net.vec_rad == 0)
         dist = dist ./ rd;
      else
         dist = dist .* rd;
      end
      dist = sum(dist,1);
      dist = exp(-dist);
      z(:,end-nrbfs+j) = dist';
   end	
end
% computes output of first layer nuerons
if (nridges > 0)
   if (rr_net.hid_type == 1)
      ztmp = 1 ./ (1 + exp(-a1));
   else
      ea1 = exp(a1);
      e_min_a1 = exp(-a1); 
      ztmp = (ea1 - e_min_a1) ./ (ea1 + e_min_a1);
   end
   z(:,2:2+nridges-1) = ztmp';
end

% computes a2
if (isfield(rr_net, 'w2_flgs'))
   a2 = (rr_net.w2 .* rr_net.w2_flgs) * z';
else
   a2 = rr_net.w2 * z';
end

% computes output
if (rr_net.output_type == 1)
   y = 1 ./ (1 + exp(-a2));
elseif (rr_net.output_type == 2)
   y = a2;
elseif (rr_net.output_type == 3)   
   ea2 = exp(a2);
   e_min_a2 = exp(-a2); 
   y = (ea2 - e_min_a2) ./ (ea2 + e_min_a2);   
end