function  test_prbfn()
%  test_prbfn()
% This function shows how to use prbfn and test it to see that all the files
% exists. The example includes preparing the data set, setting  parameters.
% Training the prbfn classifier and then using it for predication. The difference
% between regression and classification comes from the target and it is found automatically by the
% the classifier/regressor.
% The second example just show a function approximation (regresssion) problem.
% Author: Shimon Cohen
% Copyrights(C) 

% step 1: preparing the data set. 
x = [ -1 -1; 0 -1; -1 0; -2 -2; -3 -3; 0 1; 1 0; 1 1; 2 2; 3 3]';
y = [0 1; 0 1; 0 1; 0 1; 0 1; 1 0; 1 0; 1 0; 1 0 ;1 0]';
opts.verbose = 0;
% step 2: setting the parameters
opts.err_tar = 0.01;
% This output type is logsig, the second option is linear and it is 2.
opts.output_type = 1;
% perceptron hidden type in this case logsig
opts.hid_type=1;
opts.max_iters = 100;
ok = 0;
% Training x-patterns, y-targets, 2 - maximum nuber of hidden units, opts - the options parameters.
net = prbfn_train(x,y,2,opts);
ok=1;
% predication
y1 = prbfn_out(x, net);
err = sum(sum((y1 - y).*(y1 - y)));
fprintf(1,'End of test error:%f\n', err);

test_lyphoma(10);

% get the sonar data (a well known classification problem)
[p,t, pt,tt] = get_sonar;
opts.max_iters = 150;
opts.err_tar = 0.02;
net = prbfn_train(p,t,4,opts);
y1 = prbfn_out(pt,net);
[okpr,errpr] = get_class_error(tt,y1);
fprintf(1,'Sonar okprob=%f, errprob=%f\n',okpr,errpr);

%Get the vowel data set
[p,t,pt,tt] = get_vowel;
opts.err_tar = 0.25;
net = prbfn_train(p,t,22,opts);
y1 = prbfn_out(pt,net);
[okpr,errpr] = get_class_error(tt,y1);
fprintf(1,'Vowel okprob=%f, errprob=%f\n',okpr,errpr);

% Regression problem
% Set the ouput type to linear
opts.output_type = 2;
x = [-pi:0.1:pi];
y = sin(x);
data_len = length(x);
% prepare train set
perm = randperm(data_len);
train_size = round(data_len*0.66);
train_idxs = perm(1:train_size);
test_idxs = perm(train_size+1:end);
x_train = x(train_idxs);
y_train = y(train_idxs);
% prepare test set
x_test = x(test_idxs);
y_test = y(test_idxs);
% Train the regressor
net = prbfn_train(x_train,y_train,5,opts);
y1 = prbfn_out(x_train,net);
train_error = sum((y1 - y_train).*(y1 - y_train));
% predict values on test set
y2 = prbfn_out(x_test, net);
test_error = sum((y2 - y_test).*(y2 - y_test));
fprintf(1,'Regression results: train_error:%f, test_error:%f\n',train_error,test_error);

% Regression in R2 ( two features) with pruning
x = [-1:0.01:1];
x = [x;x];
y = sin(x(1,:)).*cos(x(2,:));
data_len = length(y);
train_size = round(data_len*0.66);
opts.full_evid = 0;
% prepare train set
perm = randperm(data_len);
train_size = round(data_len*0.66);
train_idxs = perm(1:train_size);
test_idxs = perm(train_size+1:end);
x_train = x(:,train_idxs);
y_train = y(train_idxs);
% prepare test set
x_test = x(:,test_idxs);
y_test = y(test_idxs);
% Train the regressor
opts.prune_val = 0.9;
% Sets optimization iterations after each weight prune.
opts.max_iters1 = 10;
opts.verbose = 1;
net = prbfn_train(x_train,y_train,5,opts);
y1 = prbfn_out(x_train,net);
train_error = sum((y1 - y_train).*(y1 - y_train));
% predict values on test set
y2 = prbfn_out(x_test, net);
test_error = sum((y2 - y_test).*(y2 - y_test));
fprintf(1,'Regression results: train_error:%f, test_error:%f\n',train_error,test_error);

%-------------------------------------------------------------------
function [okprob, errprob] = get_class_error(y1,y2)
% [okprob,errprob] = get_class_error(y1,y2)
% Description: computes classification ok probability and error probability
% and returns it in okprob and errprob.
[tdim,N] = size(y1);
okprob=0;
errprob=0;
for k=1:N
   if (tdim > 1)
      [my1,id1] = max(y1(:,k));
      [my2,id2] = max(y2(:,k));
      if (id1 == id2)
         okprob = okprob + 1;
      else
         errprob = errprob+1;
      end
   else
      if ((y1(k) > 0.5 & y2(k) > 0.5) | (y1(k)<=0.5 & y2(k)<=0.5))
         okprob = okprob + 1;
      else
         errprob = errprob+1;
      end
   end
end
okprob = okprob/N;
errprob = errprob/N;

%--------------------------------------------------------------------------
function test_lyphoma(n_tests)
file_str = 'lymphoma-multi-cv/CV_DATA_multi/lymphoma1.NO-multiclass.f';
okpr_tot = [];
errpr_tot = [];
% when reg1 = 0.01 95% okpr has been achieved
opts.reg1 = 0.01;
opts.err_tar = 0.0001;
opts.verbose = 1;
opts.output_type = 1;
% perceptron hidden type in this case logsig
opts.hid_type=1;
opts.max_iters = 150;

for j=1:n_tests
    jstr = num2str(j);
    train_name = [file_str jstr '.train'];
    test_name = [file_str jstr '.test'];
    [p,t] = read_uci_data(train_name,5);
    [pt,tt] = read_uci_data(test_name,5);
    ok = 0;
    % Training x-patterns, y-targets, 2 - maximum nuber of hidden units, opts - the options parameters.
    net = prbfn_train(p,t,5,opts);
    ok=1;
    % predication
    y1 = prbfn_out(pt, net);
    [okpr,errpr] = get_class_error(tt,y1);
    fprintf(1,'Lymphoma okprob=%f, errprob=%f\n',okpr,errpr);
    okpr_tot = [okpr_tot okpr];
    errpr_tot = [errpr_tot errpr];
end
fprintf(1,'okpr(std)\t errpr(std)\n');
for i=1:n_tests
    fprintf(1,'%f\t %f\n',okpr_tot(i), errpr_tot(i));
end
fprintf(1,'--------------------------------\n');
fprintf(1,'okpr=%f(%f), errpr=%f(%f)',mean(okpr_tot),std(okpr_tot), mean(errpr_tot), std(errpr_tot));
