function model=rbf(varargin)

% function model=rbf(varargin)
%
% Constructor for the radial basis function class,
% which is just a simple interface to Marc J.L Orrs 
% RBF toolbox for Matlab.
%
% Syntax:
% model = rbf;                  % Construct model with randomly choosen type and training method
% model = rbf('g');             % Construct model with type 'g' and randomly choosen training method
% model = rbf('c', 'rbf_rr_2'); % Construct model with type 'c' and training method rbf_rr_2
%
% Christian Merkwirth 2002

% Some "topology" parameters

switch(randsel(1:2))
	case 1
   	model.method = 'rbf_rt_1';
   case 2
      model.method = 'rbf_rt_2';
%   case 3
%      model.method = 'rbf_rr_2';
%	case 4
%      model.method = 'rbf_fs_2';
end   

switch(randsel(1:2))
	case 1
  		model.type = 'g';
   case 2
    	model.type = 'c';
   case 3
     	model.type = 'm';
end 

model.scales = randsel([0.2 0.5 1 2 4 8 10], 2);

if nargin > 1
	model.type = varargin{1}
end   
if nargin > 2
	model.method = varargin{2}
end
if nargin > 3
	model.scales = varargin{3}
end

% These fields store the trained model

model.C = [];
model.R = [];
model.w = [];
model.dmc = [];
model.conf = [];

% As usual, keep training parameters and insensitivity parameter

model.trainparams = {};		
model.eps = 0.0;

model = class(model, 'rbf');


