function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, varargin)

% function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, (sampleweights))
%
%
% Christian Merkwirth 2002

global entooldrawit

if isempty(trainparams)
   trainparams = get(model, 'trainparams');
end

nr_train = length(bilder);

if nargin > 6
   sampleweights = varargin{1};
else
   sampleweights = ones(nr_train,1);
end  

[indtrain, indtest] = dissemble(sampleclass, nr_train);

Ntrain = length(indtrain);
Ntest = length(indtest);

model.trainparams = trainparams;
model.eps = eps;

tiny = 1e-10;

trainerr = [];
testerr = [];

conf = trainparams.conf;
conf.type = model.type;
conf.scales = model.scales;

if isfield(conf, 'cen')
   conf.cen = urbilder(indtrain,:)';
end  
if isfield(conf, 'rad')
   conf.rad = (max(urbilder(indtrain,:))-min(urbilder(indtrain,:)))';
end  

trainparams.conf = conf;

[model.C, model.R, model.w, info, model.conf] = ...
   feval(model.method, urbilder(indtrain,:)', bilder(indtrain), conf);   

model.dmc = info.dmc;

solltrain = bilder(indtrain);
isttrain = calc(model, urbilder(indtrain,:));
trainerr(end+1,1) = relepsinloss(solltrain, isttrain, eps, sampleweights(indtrain));

if Ntest   
	solltest = bilder(indtest);
	isttest = calc(model, urbilder(indtest,:));
   
	testerr(end+1,1) = relepsinloss(solltest, isttest, eps, sampleweights(indtest));
end   
   
if entooldrawit      
     	subplot(1,2,1)
     	plot(solltrain, isttrain, '.') 
      hold on
      if Ntest
      	plot(solltest, isttest, 'r.') 
      end
      
      hold off
      
      
      subplot(1,2,2)
      plot(1:Ntrain,solltrain);
      hold on
      plot(1:Ntrain,isttrain, 'r');
      
      if Ntest 
      	plot((Ntrain+1):(Ntrain+Ntest), solltest, 'g');
      	plot((Ntrain+1):(Ntrain+Ntest), isttest, 'r');      
      end   
      
      hold off   
      drawnow
end   

