function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, varargin)

% function [model, trainerr] = train(model, urbilder, bilder, sampleclass, trainparams, eps, (sampleweights))
%
%
% Christian Merkwirth 2002

global entooldrawit

if isempty(trainparams)
   trainparams = get(model, 'trainparams');
end

nr_samples = length(bilder);

if nargin > 6
   sampleweights = varargin{1};
else
   sampleweights = ones(nr_samples,1);
end  

[indtrain, indtest] = dissemble(sampleclass, nr_samples);

nr_train = length(indtrain);
nr_test = length(indtest);

% Limits for the number of nearest neighbors

klimit = ceil(0.7*nr_train);

if length(model.kmax) == 1
   krange = (1:min(model.kmax, klimit))+1;
else
   krange = (min(model.kmax(1), klimit):min(model.kmax(2), klimit))+1;
end

model.kmax = [min(krange) max(krange)];

D = size(urbilder,2);       % dimension of inputs
nr_new_sugg = trainparams.initpopsize;

errmin = [];
errmed = [];
errtest = [];

metrics = newmetric(nr_new_sugg, D, trainparams);
k = min(krange);

model.trainparams = trainparams;
model.eps = eps;

tiny = 1e-10;

for o=1:max(1,trainparams.outer)     
   errs = [];
   models = model;
   
   for i=1:size(metrics,1)
      [e, m] = calcmodel(model, urbilder(indtrain,:), bilder(indtrain), metrics(i,:), k, ... 
          eps, sampleweights(indtrain));
      models(i,1) = m;
      errs(i,1) = e;
   end
   
   [err, indbest] = min(errs);
   errmed(end+1) = median(errs);
   
   [k, models(indbest), err, solltrain, isttrain] = choosek(models(indbest), krange, eps, sampleweights(indtrain));   
   errmin(end+1) = err;
   errs(indbest) = err;
   
   if length(indtest)
      [errtest(end+1), solltest, isttest] = calctesterror(models(indbest), urbilder(indtest,:), bilder(indtest), ...
         					eps, sampleweights(indtest));
   else
      errtest = [];
      solltest = [];
      isttest = [];
   end   
      
   if entooldrawit      
      subplot(2,2,1)
      plot(solltrain, isttrain, '.')
      hold on
      plot(solltest, isttest, 'r.')    
      hold off
      
      subplot(2,2,3)
      plot(1:nr_train, solltrain)
      hold on
      plot(1:nr_train, isttrain, 'r')
      plot((1+nr_train):(nr_train+nr_test), solltest, 'g')
      plot((1+nr_train):(nr_train+nr_test), isttest, 'r')
      hold off
      title(num2str(errmin(end)))
      subplot(2,2,4)
      imagesc(metrics')
      colorbar
      title(num2str(models(indbest).k))
      subplot(2,2,2)
      
      semilogy(errmin+tiny, 'b')
      hold on
      semilogy(errmed+tiny, 'k')
      if length(indtest)
      	semilogy(errtest+tiny, 'r')
      end   
      hold off

      drawnow       
   end
   
   if (err == 0)
   	break   
   end
   
   if (size(metrics, 1) < max([trainparams.nr_models 4])) 
   	break;   
   end
   
   metrics = metricga(metrics, errs, trainparams);   
   
   metrics = cleanmetric(metrics);
   metrics = normalizemetric(metrics, trainparams);
   metrics = cleanmetric(metrics);
end

trainerr = errmin;

[dummy,ind] = sort(errs);
model = models(ind(1:trainparams.nr_models));


function [err, model] = calcmodel(model, urbilder, bilder, metric, k, eps, varargin)   

model.metric =  metric;
model.images = bilder;
model.dataset = urbilder .* repmat(model.metric, size(urbilder, 1), 1);

model.searcher = nn_prepare(model.dataset, model.metricclass);
model.k = k;

y = calc(model, []);

%if any(nn(:,1) == refind)
%   warning('Self matches allowed')
%end

err = relepsinloss(bilder, y, eps, varargin{:});


function [k, model, err, solltrain, isttrain] = choosek(model, krange, eps, varargin)

% Determine optimal number of neighbors on training set

model.k = max(krange);
[nn, d, querypoints] = calcnn(model, []);

kerrs = [];
ym = [];

for k=krange
   y = calcoutput(model, nn(:,1:k), d(:,1:k), querypoints);
   kerrs(end+1) = relepsinloss(model.images, y, eps, varargin{:});
   ym(:,end+1) = y; 
end   

%plot(krange,kerrs);
%pause

[err, ind] = min(kerrs);
k = krange(ind);
model.k = k;
solltrain = model.images;
isttrain = ym(:,ind);


function [err, soll, ist] = calctesterror(model, urbilder, bilder, eps, varargin)

% Calculate error on test set

soll = bilder;
ist = calc(model, urbilder);
err = relepsinloss(soll, ist, eps, varargin{:});


% Now functions to handle metric

function metrics = newmetric(N,D, trainparams)

metrics = [];

while(isempty(metrics))
    metrics = zeros(N,D);
	metrics_set = floor(rand(N,D)*trainparams.nr_quants)/(trainparams.nr_quants-1);
	
    ind = find(rand(size(metrics)) < ((D.^trainparams.dimension_exponent)/D));
	metrics(ind) = metrics_set(ind);
   
   metrics = cleanmetric(metrics);
   metrics = normalizemetric(metrics, trainparams);
   metrics = cleanmetric(metrics);
end

   
function metrics = normalizemetric(metrics, trainparams)

if ~isempty(metrics)
	metrics = metrics ./ repmat(max(metrics, [],2), 1, size(metrics,2));
	metrics = metrics * (trainparams.nr_quants-1);
	metrics = round(metrics);
	metrics = metrics / (trainparams.nr_quants-1);
end

function m = cleanmetric(metrics)

% Clean matrix metrics from rows which are all zero and from duplicate rows

ind = find(~all((metrics == 0),2));
m = unique(metrics(ind,:), 'rows');

