%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%
%%
%%  Demo for Prediction of Time-Series
%%
%%
%%  Joerg Wichard 2002

%%  Flag for graphical output
global entooldrawit
entooldrawit = 1;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Preprocessing the data

%%Loading the Data 
data  = load('Laser.dat');

dim = 10;    %% Embedding Dimension
lag = 1;     %% Time-Lag
split = 900; %% Divide the data in training and OOS

%% Generating the image/preimage-data via time-delay-embedding
tmpd     = delay_embed(data,(1+dim),lag);
urbilder = tmpd(1:end,1:dim);
bilder   = tmpd(1:end,(1+dim));

%% Splitting the data in training set and a test set
urbilder_train = urbilder(1:split,:);
bilder_train   = bilder(1:split);

urbilder_test = urbilder(split:end,:);
bilder_test   = bilder(split:end);

%%% Setting the Parameters
ens = crosstrainensemble;
enstrainparams = get(ens, 'trainparams');
eps = 0.01;

%% Select the classes that should form the ensemble
enstrainparams.modelclasses = {...
    'linear', [], {}; ...
    'vicinal', [], {}; ...
    'perceptron', [], {}; ...
%    'polynomial', [], {}; ...
		   };

%% Training the ensemble
ens = train(ens, urbilder_train, bilder_train, [], enstrainparams, eps); 

%% Predicting the unseen data points
[zm, z] = calc(ens, urbilder_test);

%% Iterated Predition of the unseen data points
tmp_bilder = bilder_train; 
tmp_states = delay_embed(tmp_bilder,dim,lag);

for i=1:length(bilder_test)
    
  tmp_states     = delay_embed(tmp_bilder,dim,lag);
  actual_state   = tmp_states(end,:);
  
  [zm, z] = calc(ens, actual_state);
  
  tmp_bilder = [ tmp_bilder; zm ];
end

figure(2)
clf;
plot(tmp_bilder,'r');
hold on;
plot(bilder)
legend('Iterated Prediction','Training Data');
hold off;