% Demo script for learning a simple two-dimensional function,
% the user can choose between Matlab's peaks function and the
% ramp-hill function.
% 
% Christian Merkwirth and JDW 2003

% Turn on visualization of learning progress
global entooldrawit 
entooldrawit = 1;

N = 100;			% number of observations available for training

%% Select the examble

problem = 'peaks';
%problem = 'ramp';

add_noisy_dimensions = 0;

%% Generate data
switch(lower(problem))
 case 'peaks' 
  X = 6 * rand(N,1) - 3;
  Y = 6 * rand(N,1) - 3;

  Z = peaks(X,Y);

  [urbilder, preimagescalefactors] = scale([X(:) Y(:)]);
  [bilder, imagescalefactors] = scale(Z(:));
 case 'ramp'
  [urbilder, bilder] = ramp_hill(N);
end   

%% Add artificial noise
if add_noisy_dimensions
  urbilder = [urbilder triangle_rand(size(urbilder,1),5)];
end

% Add noise to the observations
bilder = bilder + 0.1*std(bilder)*randn(size(bilder));

%% The epsilon insensitive error loss
eps = 0.05;

%% Select the ensemble class
ens = crosstrainensemble;

%% Get the parameters of that class
enstrainparams = get(ens, 'trainparams');

%% Don't use extra weights when building the ensemble
enstrainparams.weight_models = 0;

%% Select the classes that should form the ensemble
enstrainparams.modelclasses = {'vicinal', [], {}; 'perceptron', [], {}; 'perceptron2', [], {};};
    
%% Take 10 training partitions for cross validation
enstrainparams.nr_cv_partitions = 10;

%% Train the ensemble
ens = train(ens, urbilder, bilder, [], enstrainparams, eps); 


switch(lower(problem))
 case 'peaks'
  [x0, y0, z0] = peaks;

  urcomp = scale([x0(:) y0(:)], preimagescalefactors);
  zcomp = scale(z0(:), imagescalefactors);
  
  M = size(urcomp,1);
  x0 = reshape(urcomp(:,1), sqrt(M),sqrt(M));
  y0 = reshape(urcomp(:,2), sqrt(M),sqrt(M));

  
 case 'ramp'
  [urcomp, zcomp] = ramp_hill;
  
  M = size(urcomp,1);
  x0 = reshape(urcomp(:,1), sqrt(M),sqrt(M));
  y0 = reshape(urcomp(:,2), sqrt(M),sqrt(M));
end


if add_noisy_dimensions
  urcomp = [urcomp triangle_rand(size(urcomp,1),5)];
end

[zm, z] = calc(ens, urcomp);


generrs = [];

for i=1:size(z,2)
  generrs(i) = relepsinloss(zcomp(:), z(:,i),0.0);
end

generrensemble = relepsinloss(zcomp(:), zm,0.0);

disp(['Mean of individual generalization errors ' num2str(mean(generrs))])
disp(['Best individual generalization error ' num2str(min(generrs))])
disp(['Ensemble generalization error ' num2str(generrensemble)])



%%% Plot the results

figure(1)
clf;

subplot(2,2,1)
mesh(x0, y0, reshape(zm, size(x0)))
hold on
plot3(urbilder(:,1), urbilder(:,2), bilder, 'k*')
title('Output of the trained ensemble');
hold off
axis tight
rotate3d on

subplot(2,2,2)
surf(x0, y0, reshape(zcomp(:), size(x0)))
title('Original Function');
axis tight
rotate3d on

subplot(2,2,3)
surf(x0, y0, (reshape(zcomp(:), size(x0)) - reshape(zm, size(x0)) ));
title('Absolute error of the ensemble');
axis tight
rotate3d on

subplot(2,2,4)
contour(x0, y0, reshape(zcomp(:), size(x0)) );
hold on
grid on
plot(urbilder(:,1), urbilder(:,2),'k*')
title('Distribution of the training samples');
hold off
axis square
