function cc = matthews(y,z)

% function cc = matthews(y,z)
%
% Compute Matthews correlation coefficient for binary classification.
% y and z are binary vectors, y is the desired, z is the actual output.
%
% Christian Merkwirth MPI 2002

y = round(y(:));
z = round(z(:));

P = sum(double(y & z));     % number of true positives
N = sum(double((~y) & (~z)));   % number of true negatives
O = sum(double((~y) & (z)));   % number of false positives
U = sum(double((y) & (~z)));   % number of false negatives

n = sqrt((N+O)*(N+U)*(P+O)*(P+U));

cc = (N*P-O*U);

if (n == 0)
   cc = 0; 
else
   cc = cc / n;
end
