#include <stdio.h>
#include <vector.h>
#include <algo.h>

#include "mextools/mextools.h"

typedef pair<double, int> valpos;

struct myless : public binary_function<valpos, valpos, bool>  {
	bool operator()(const valpos& x, const valpos& y) const { 
		return (x.first < y.first);
	}
};

void mexFunction(int nlhs, mxArray  *plhs[], int nrhs, const mxArray  *prhs[])       
{                   
	if (nrhs < 2) {
		mexErrMsgTxt("weightedmedian(x, weights)\n");
		return;
	}
           
  	mmatrix x(prhs[0]);
	mvector weights(prhs[1]);

	const long nr_samples = x.getM();
	const long nr_dim = weights.length();
	
	if (x.getN() != weights.length()) {
		mexErrMsgTxt("Number of weights and number of data columns does not match!\n");
		return;
	}
	
	plhs[0] = mxCreateDoubleMatrix(nr_samples,1, mxREAL);  
	mvector out(plhs[0]);	

	double total_sum = 0.0;

	for (int i=1; i <= nr_dim; i++) {
		total_sum += weights(i);	
		if (weights(i) < 0) {
			mexErrMsgTxt("Negative weight given!\n");
			return;
		}
	}

	if (total_sum <= 0) {
		mexErrMsgTxt("Sum of weights must be positive!\n");
		return;
	}
	
	const double thresh = total_sum / 2.0;
	
	for (int j=1; j <= nr_samples; j++) {
		vector<valpos> v(nr_dim);
		
		for (int i=1; i <= nr_dim; i++) {
			v[i-1] = make_pair(x(j,i), i);
		}
		
		sort(v.begin(),v.end());
		
		double cumsum = 0.0;
		
		for (int i=0; i < nr_dim; i++) {
			cumsum += weights(v[i].second);
			
			if (cumsum == thresh) {
				if (i < (nr_dim-1))
					out(j) = 0.5 * (v[i].first + v[i+1].first);
				else
					out(j) = v[i].first;
				
				break;
			} else if (cumsum > thresh) {
				out(j) = v[i].first;
				break;
			}								
		}		
	}		
}
